/*
 * SYNOPSIS:
 *	dlalmcrcv device ppa sap
 *
 *	Receive with all Multicast Addresses enabled.
 */

#include	<sys/types.h>
#include	<sys/stropts.h>
#include	<sys/dlpi.h>
#include	<stdio.h>
#include	<sys/ethernet.h>
#include	"dltest.h"

#define		MAXDATA		(32 * 1024)
#define		MAXCTL		(4 * 1024)

/*
 * Xmit data buffer.  Fill with whatever pattern you like ...
 */
u_char	databuf[MAXDATA];
u_char	ctlbuf[MAXCTL];

extern	char	*optarg;
extern	int	optind;

main(ac, av)
int	ac;
char	*av[];
{
	char	*device;
	int	ppa;
	int	fd;
	int	sap;
	struct	strbuf	ctl, data;
	int	flags;
	int	i;
	int	c;
	dl_unitdata_ind_t	*unitdataindp;
	struct	dladdr	*addrp;
	int	offset;
	int	len;
	long	buf[MAXDLBUF];

	setbuf(stdout, NULL);

	if (ac < 4)
		usage(av[0]);

	if ((ac - optind) != 3)
		usage(av[0]);

	device = av[optind++];
	ppa = atoi(av[optind++]);
	sap	= atoi(av[optind++]);

	/*
	 * Validate arguments.
	 */
	if (ppa < 0)
		err("ppa cannot be negative");
	if (sap < 0)
		err("sap cannot be negative");

	/*
	 * Open the device.
	 */
	if ((fd = open(device, 2)) < 0)
		syserr(device);

	/*
	 * Attach.
	 */
	dlattachreq(fd, ppa);

	/*
	 * Wait for DL_OK_ACK response.
	 */
	dlokack(fd, buf);

	/*
	 * Bind.
	 */
	dlbindreq(fd, sap, 0, DL_CLDLS, 0, 0);

	/*
	 * Wait for DL_BIND_ACK response.
	 */
	dlbindack(fd, buf);

	/*
	 * Turn on the all-multicast promiscuous mode.
	 */
	dlpromisconreq(fd, DL_PROMISC_MULTI);

	/*
	 * Wait for DL_OK_ACK response.
	 */
	dlokack(fd, buf);

	/*
	 * Read packets.
	 */

	ctl.buf = (char *) ctlbuf;
	ctl.maxlen = MAXCTL;
	ctl.len = 0;

	data.buf = (char *) databuf;
	data.maxlen = MAXDATA;
	data.len = 0;

	unitdataindp = (dl_unitdata_ind_t *) ctlbuf;

	while (getmsg(fd, &ctl, &data, &flags) == 0) {

		if (unitdataindp->dl_primitive != DL_UNITDATA_IND)
			printf("bad primitive:  %d\n",
				unitdataindp->dl_primitive);

		offset = unitdataindp->dl_dest_addr_offset;
		len = unitdataindp->dl_dest_addr_length - sizeof (long);
		addrp = (struct dladdr *) OFFADDR(ctlbuf, offset);
		printf("dest addr ");
		for (i = 0; i < 6; i++)
			printf("%x ",
				addrp->dl_phys.ether_addr_octet[i]);
		printf("sap %d ", addrp->dl_sap);

		offset = unitdataindp->dl_src_addr_offset;
		len = unitdataindp->dl_src_addr_length - sizeof (long);
		addrp = (struct dladdr *) OFFADDR(ctlbuf, offset);
		printf("src addr ");
		for (i = 0; i < 6; i++)
			printf("%x ",
				addrp->dl_phys.ether_addr_octet[i]);
		printf("sap %d\n", addrp->dl_sap);

		ctl.len = 0;
		data.len = 0;
	}
}

usage(av0)
char	*av0;
{
	(void) fprintf(stderr, "%s:  ", av0);
	(void) fprintf(stderr, "Usage:  %s [ -p ] device ppa sap\n", av0);
	(void) exit(1);
}
