/*
 * NAME
 *  dlarp - read and display raw datalink packets from multiple devices
 * SYNOPSIS:
 *	dlarp device nppa times
 *
 * Open datalink provider.
 * Attach to PPA.
 * Bind to sap
 * Issue DLIOCRAW ioctl.
 * Read and print messages
 */

#include	<sys/types.h>
#include	<sys/time.h>
#include	<sys/stream.h>
#include	<sys/signal.h>
#include	<sys/stropts.h>
#include	<sys/dlpi.h>
#include	<sys/ethernet.h>
#include	<stdio.h>
#include	"dltest.h"

long	databuf[MAXDLBUF];

extern	char	*optarg;
extern	int	optind;

int	nppa;
int	fd[100];
struct	pollfd	pollfds[100];

struct  arp_header {
        u_short ar_hrd;		/* format of hardware address */
        u_short ar_pro;		/* format of protocol address */
        u_char  ar_hln;		/* length of hardware address */
        u_char  ar_pln;		/* length of protocol address */
        u_short ar_op;		/* opcode */
        u_char  ar_sha[6];	/* sender hardware address */
        u_char  ar_spa[4];	/* sender protocol address */
        u_char  ar_tha[6];	/* target hardware address */
        u_char  ar_tpa[4];	/* target protocol address */
};
struct  arp_header arh = {
	0x0001,
	0x0800,
	0x06,
	0x04,
	0x0001,
	{0x08, 0x00, 0x20, 0x09, 0xa2, 0xaf},
	{0x81, 0x90, 0x88, 0x69},
	{0,0,0,0,0,0},
	{0x81, 0x90, 0x0a, 0x18}
};

char    xmitbuf[MAXDLBUF];

main(ac, av)
int	ac;
char	*av[];
{
	long	buf[MAXDLBUF];
	char	*device;
	int	sap;
	int	i;
	int	j;
	int	k;
	int	l;
	int	c;
	int	offset;
	int	len;
	struct	timeval	t;
	u_int	chunksize = 16 * 1024;
        u_char  phys[ETHERADDRL] = {0xff, 0xff, 0xff, 0xff, 0xff, 0xff};
	int     physlen = 6;
	struct	strbuf	data;
	char	*p, *limp;
	int	flags;
	struct	sb_hdr	*bp;
	int	tfd;
	int	size;
	int	times;
        struct  ether_header    *ehp;
	struct	arh_s	*arhp;
        union   DL_primitives   *dlp;

        device = av[optind++];
	nppa = atoi(av[optind++]);
	sap = 2054;
	size = 42;
	times = atoi(av[optind++]);
printf("device %s nppa %d times %d\n", device, nppa, times);

	/*
	 * Validate arguments.
	 */
	if (nppa < 0)
		err("nppa cannot be negative");

	for (i = 0; i < nppa; i++) {

		/*
		 * Open the device.
		 */
		if ((fd[i] = open(device, 2)) < 0)
			syserr(device);

		/*
		 * Attach.
		 */
		dlattachreq(fd[i], i);
		dlokack(fd[i], buf);

		/*
		 * Bind.
		 */
		dlbindreq(fd[i], sap, 0, DL_CLDLS, 0, 0);
		dlbindack(fd[i], buf);

		/*
		 * Issue DLIOCRAW ioctl.
		 */
		if (strioctl(fd[i], DLIOCRAW, -1, 0, NULL) < 0)
			syserr("DLIOCRAW");

		/*
		 * Flush the read side of the Stream.
		 */
		if (ioctl(fd[i], I_FLUSH, FLUSHR) < 0)
			syserr("I_FLUSH");

		pollfds[i].fd = fd[i];
		pollfds[i].events = POLLIN;
	}

	/*
	 * Open the device for xmit.
	 */
	if ((tfd = open(device, 2)) < 0)
		syserr(device);

	/*
	 * Attach.
	 */
	dlattachreq(tfd, 0);
	dlokack(tfd, buf);

	/*
	 * Bind.
	 */
	dlbindreq(tfd, sap, 0, DL_CLDLS, 0, 0);
	dlbindack(tfd, buf);

	/*
	 * Get our physical address.
	 */
	dlphysaddrreq(tfd, DL_CURR_PHYS_ADDR);
	dlphysaddrack(tfd, buf);
	dlp = (union DL_primitives*) buf;
 
	ehp = (struct ether_header*) xmitbuf;
	memcpy(&ehp->ether_dhost, phys, ETHERADDRL);
	memcpy(&ehp->ether_shost, OFFADDR(dlp, 
		dlp->physaddr_ack.dl_addr_offset), ETHERADDRL);  
	ehp->ether_type = (u_short) sap;
	memcpy(xmitbuf + sizeof(struct ether_header), &arh, 
		sizeof(struct arp_header));

	if (strioctl(tfd, DLIOCRAW, -1, 0, 0) < 0)
		syserr("ioctl DLIOCRAW");

	/*
	 * Xmit the frames.
	 */
	for (j = 0; j < times; j++)
		if (write(tfd, xmitbuf, size) < 0)
			syserr("write");

	for (;;) {
		if (poll(pollfds, nppa, -1) < 0) {
			perror("poll failed");
			exit(3);
		}

		for (i = 0; i < nppa; i++) {
			if (pollfds[i].revents == POLLIN) {

				/*
				 * Read packets.
				 */
				data.buf = (char *) databuf;
				data.maxlen = MAXDLBUF;
				data.len = 0;

				getmsg(fd[i], NULL, &data, &flags);

				limp = p + data.len;
				if (data.len) {
					printf("dest addr ");
					for (k = 0; k < 6; k++)
						printf("%02x ", data.buf[k] & 0xff);
					printf("src addr ");
					for (k = 6; k < 12; k++)
						printf("%02x ", data.buf[k] & 0xff);
					printf("ether type ");
					for (k = 12; k < 14; k++)
						printf("%02x ", data.buf[k] & 0xff);
					printf("\n");
				}
				data.len = 0;
			}
		}
	}
}
