/*
 * NAME
 *  dlrcv - read and display raw datalink packets
 * SYNOPSIS:
 *	dlrcv [ -bp ] device ppa sap
 *
 * Open datalink provider.
 * Attach to PPA.
 * Bind to sap
 * Issue DLIOCRAW ioctl.
 * Read and print messages
 */

/*
typedef	unsigned long ulong;
*/

#include	<sys/types.h>
#include	<sys/time.h>
#include	<sys/stream.h>
#include	<sys/stropts.h>
#include	<sys/dlpi.h>
#include	<sys/bufmod.h>
#include	<stdio.h>
#include	"dltest.h"

#define	BUFFERSIZE	(32 * 1024)

long	databuf[BUFFERSIZE];

extern	char	*optarg;
extern	int	optind;

int	promisc = 1;	/* promiscuous mode "off" by default */
int	bufmod = 1;		/* push buffer module, "off" by default */

main(ac, av)
int	ac;
char	*av[];
{
	long	buf[BUFFERSIZE];
	char	*device;
	int	ppa;
	int	fd;
	int	sap;
	struct	strbuf	data;
	int	flags;
	int	i;
	int	c;
	int	offset;
	int	len;
	struct	timeval	t;
	u_int	chunksize = BUFFERSIZE/2;
	struct	sb_hdr	*bp;
	char	*p, *limp;
	int	snaplen;

	if (ac < 4)
		usage(av[0]);

	device = av[1];
	ppa = atoi(av[2]);
	sap	= atoi(av[3]);
	snaplen = atoi(av[4]);

	/*
	 * Validate arguments.
	 */
	if (ppa < 0)
		err("ppa cannot be negative");
	if (sap < 0)
		err("sap cannot be negative");

	/*
	 * Open the device.
	 */
	if ((fd = open(device, 2)) < 0)
		syserr(device);

	/*
	 * Attach.
	 */
	dlattachreq(fd, ppa);
	dlokack(fd, buf);

	dlpromisconreq(fd, DL_PROMISC_PHYS);
	dlokack(fd, buf);

	/*
	 * Bind.
	 */
	dlbindreq(fd, sap, 0, DL_CLDLS, 0, 0);
	dlbindack(fd, buf);

	/*
	 * Issue DLIOCRAW ioctl.
	 */
	if (strioctl(fd, DLIOCRAW, -1, 0, NULL) < 0)
		syserr("DLIOCRAW");

	/*
	 * Push and configure buffer module.
	 */
	if (ioctl(fd, I_PUSH, "bufmod") < 0)
		syserr("push bufmod");

	t.tv_sec = 0;
	t.tv_usec = 500000;	/* 0.5s */
	if (strioctl(fd, SBIOCSTIME, -1, sizeof (struct timeval),
		&t) < 0)
		syserr("SBIOCSTIME");
	if (strioctl(fd, SBIOCSCHUNK, -1, sizeof (u_int),
		&chunksize) < 0)
		syserr("SBIOCSCHUNK");
	if (strioctl(fd, SBIOCSSNAP, -1, sizeof (u_long),
		&snaplen) < 0)
		syserr("SBIOCSSNAP");

	/*
	 * Flush the read side of the Stream.
	 */
	if (ioctl(fd, I_FLUSH, FLUSHR) < 0)
		syserr("I_FLUSH");

	/*
	 * Read packets.
	 */

	data.buf = (char *) databuf;
	data.maxlen = BUFFERSIZE;
	data.len = 0;

	while (getmsg(fd, NULL, &data, &flags) == 0) {

		limp = data.buf + data.len;

		if (data.len) {
			for (p = data.buf; p < limp; p += bp->sbh_totlen) {
				bp = (struct sb_hdr*) p;
				for (i = 0; i < bp->sbh_totlen; i++)
					printf("%02x ", *(p + sizeof (struct sb_hdr)
						+ i) & 0xff);
				printf("\n");
			}
printf("\n");
		}

		data.len = 0;
	}
}

usage(av0)
char	*av0;
{
	(void) fprintf(stderr, "%s:  ", av0);
	(void) fprintf(stderr, "Usage:  %s device ppa sap snaplen\n", av0);
	(void) exit(1);
}
